@@ -14,22 +14,22 @@ use tokio::time::Instant;
 
 #[async_trait]
 trait ControllingSelector {
-    fn start(&mut self);
-    async fn contact_candidates(&mut self);
+    async fn start(&self);
+    async fn contact_candidates(&self);
     async fn ping_candidate(
-        &mut self,
+        &self,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
     );
     async fn handle_success_response(
-        &mut self,
+        &self,
         m: &Message,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
         remote_addr: SocketAddr,
     );
     async fn handle_binding_request(
-        &mut self,
+        &self,
         m: &Message,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
@@ -38,22 +38,22 @@ trait ControllingSelector {
 
 #[async_trait]
 trait ControlledSelector {
-    fn start(&mut self);
-    async fn contact_candidates(&mut self);
+    async fn start(&self);
+    async fn contact_candidates(&self);
     async fn ping_candidate(
-        &mut self,
+        &self,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
     );
     async fn handle_success_response(
-        &mut self,
+        &self,
         m: &Message,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
         remote_addr: SocketAddr,
     );
     async fn handle_binding_request(
-        &mut self,
+        &self,
         m: &Message,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
@@ -62,21 +62,22 @@ trait ControlledSelector {
 
 impl AgentInternal {
     async fn is_nominatable(&self, c: &Arc<dyn Candidate + Send + Sync>) -> bool {
+        let start_time = self.start_time.lock().await;
         match c.candidate_type() {
             CandidateType::Host => {
-                Instant::now().duration_since(self.start_time).as_nanos()
+                Instant::now().duration_since(*start_time).as_nanos()
                     > self.host_acceptance_min_wait.as_nanos()
             }
             CandidateType::ServerReflexive => {
-                Instant::now().duration_since(self.start_time).as_nanos()
+                Instant::now().duration_since(*start_time).as_nanos()
                     > self.srflx_acceptance_min_wait.as_nanos()
             }
             CandidateType::PeerReflexive => {
-                Instant::now().duration_since(self.start_time).as_nanos()
+                Instant::now().duration_since(*start_time).as_nanos()
                     > self.prflx_acceptance_min_wait.as_nanos()
             }
             CandidateType::Relay => {
-                Instant::now().duration_since(self.start_time).as_nanos()
+                Instant::now().duration_since(*start_time).as_nanos()
                     > self.relay_acceptance_min_wait.as_nanos()
             }
             CandidateType::Unspecified => {
@@ -89,56 +90,68 @@ impl AgentInternal {
         }
     }
 
-    async fn nominate_pair(&mut self) {
-        if let Some(pair) = &self.nominated_pair {
-            // The controlling agent MUST include the USE-CANDIDATE attribute in
-            // order to nominate a candidate pair (Section 8.1.1).  The controlled
-            // agent MUST NOT include the USE-CANDIDATE attribute in a Binding
-            // request.
+    async fn nominate_pair(&self) {
+        let result = {
+            let nominated_pair = self.nominated_pair.lock().await;
+            if let Some(pair) = &*nominated_pair {
+                // The controlling agent MUST include the USE-CANDIDATE attribute in
+                // order to nominate a candidate pair (Section 8.1.1).  The controlled
+                // agent MUST NOT include the USE-CANDIDATE attribute in a Binding
+                // request.
 
-            let (msg, result) = {
-                let username = self.remote_ufrag.clone() + ":" + self.local_ufrag.as_str();
-                let mut msg = Message::new();
-                let result = msg.build(&[
-                    Box::new(BINDING_REQUEST),
-                    Box::new(TransactionId::new()),
-                    Box::new(Username::new(ATTR_USERNAME, username)),
-                    Box::new(UseCandidateAttr::default()),
-                    Box::new(AttrControlling(self.tie_breaker)),
-                    Box::new(PriorityAttr(pair.local.priority())),
-                    Box::new(MessageIntegrity::new_short_term_integrity(
-                        self.remote_pwd.clone(),
-                    )),
-                    Box::new(FINGERPRINT),
-                ]);
-                (msg, result)
-            };
+                let (msg, result) = {
+                    let ufrag_pwd = self.ufrag_pwd.lock().await;
+                    let username =
+                        ufrag_pwd.remote_ufrag.clone() + ":" + ufrag_pwd.local_ufrag.as_str();
+                    let mut msg = Message::new();
+                    let result = msg.build(&[
+                        Box::new(BINDING_REQUEST),
+                        Box::new(TransactionId::new()),
+                        Box::new(Username::new(ATTR_USERNAME, username)),
+                        Box::new(UseCandidateAttr::default()),
+                        Box::new(AttrControlling(self.tie_breaker.load(Ordering::SeqCst))),
+                        Box::new(PriorityAttr(pair.local.priority())),
+                        Box::new(MessageIntegrity::new_short_term_integrity(
+                            ufrag_pwd.remote_pwd.clone(),
+                        )),
+                        Box::new(FINGERPRINT),
+                    ]);
+                    (msg, result)
+                };
 
-            if let Err(err) = result {
-                log::error!("{}", err);
+                if let Err(err) = result {
+                    log::error!("{}", err);
+                    None
+                } else {
+                    log::trace!(
+                        "ping STUN (nominate candidate pair from {} to {}",
+                        pair.local,
+                        pair.remote
+                    );
+                    let local = pair.local.clone();
+                    let remote = pair.remote.clone();
+                    Some((msg, local, remote))
+                }
             } else {
-                log::trace!(
-                    "ping STUN (nominate candidate pair from {} to {}",
-                    pair.local,
-                    pair.remote
-                );
-                let local = pair.local.clone();
-                let remote = pair.remote.clone();
-                self.send_binding_request(&msg, &local, &remote).await;
+                None
             }
+        };
+
+        if let Some((msg, local, remote)) = result {
+            self.send_binding_request(&msg, &local, &remote).await;
         }
     }
 
-    pub(crate) fn start(&mut self) {
-        if self.is_controlling {
-            ControllingSelector::start(self);
+    pub(crate) async fn start(&self) {
+        if self.is_controlling.load(Ordering::SeqCst) {
+            ControllingSelector::start(self).await;
         } else {
-            ControlledSelector::start(self);
+            ControlledSelector::start(self).await;
         }
     }
 
-    pub(crate) async fn contact_candidates(&mut self) {
-        if self.is_controlling {
+    pub(crate) async fn contact_candidates(&self) {
+        if self.is_controlling.load(Ordering::SeqCst) {
             ControllingSelector::contact_candidates(self).await;
         } else {
             ControlledSelector::contact_candidates(self).await;
@@ -146,11 +159,11 @@ impl AgentInternal {
     }
 
     pub(crate) async fn ping_candidate(
-        &mut self,
+        &self,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
     ) {
-        if self.is_controlling {
+        if self.is_controlling.load(Ordering::SeqCst) {
             ControllingSelector::ping_candidate(self, local, remote).await;
         } else {
             ControlledSelector::ping_candidate(self, local, remote).await;
@@ -158,13 +171,13 @@ impl AgentInternal {
     }
 
     pub(crate) async fn handle_success_response(
-        &mut self,
+        &self,
         m: &Message,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
         remote_addr: SocketAddr,
     ) {
-        if self.is_controlling {
+        if self.is_controlling.load(Ordering::SeqCst) {
             ControllingSelector::handle_success_response(self, m, local, remote, remote_addr).await;
         } else {
             ControlledSelector::handle_success_response(self, m, local, remote, remote_addr).await;
@@ -172,12 +185,12 @@ impl AgentInternal {
     }
 
     pub(crate) async fn handle_binding_request(
-        &mut self,
+        &self,
         m: &Message,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
     ) {
-        if self.is_controlling {
+        if self.is_controlling.load(Ordering::SeqCst) {
             ControllingSelector::handle_binding_request(self, m, local, remote).await;
         } else {
             ControlledSelector::handle_binding_request(self, m, local, remote).await;
@@ -187,24 +200,35 @@ impl AgentInternal {
 
 #[async_trait]
 impl ControllingSelector for AgentInternal {
-    fn start(&mut self) {
-        self.start_time = Instant::now();
-        self.nominated_pair = None;
+    async fn start(&self) {
+        {
+            let mut nominated_pair = self.nominated_pair.lock().await;
+            *nominated_pair = None;
+        }
+        {
+            let mut start_time = self.start_time.lock().await;
+            *start_time = Instant::now();
+        }
     }
 
-    async fn contact_candidates(&mut self) {
+    async fn contact_candidates(&self) {
         // A lite selector should not contact candidates
-        if self.lite {
+        if self.lite.load(Ordering::SeqCst) {
             // This only happens if both peers are lite. See RFC 8445 S6.1.1 and S6.2
             log::trace!("now falling back to full agent");
         }
 
+        let nominated_pair_is_some = {
+            let nominated_pair = self.nominated_pair.lock().await;
+            nominated_pair.is_some()
+        };
+
         if self.agent_conn.get_selected_pair().await.is_some() {
             if self.validate_selected_pair().await {
-                log::trace!("checking keepalive");
+                log::trace!("[{}]: checking keepalive", self.get_name());
                 self.check_keepalive().await;
             }
-        } else if self.nominated_pair.is_some() {
+        } else if nominated_pair_is_some {
             self.nominate_pair().await;
         } else {
             let has_nominated_pair =
@@ -222,7 +246,10 @@ impl ControllingSelector for AgentInternal {
                         p.remote.to_string()
                     );
                     p.nominated.store(true, Ordering::SeqCst);
-                    self.nominated_pair = Some(p);
+                    {
+                        let mut nominated_pair = self.nominated_pair.lock().await;
+                        *nominated_pair = Some(p);
+                    }
                 }
 
                 self.nominate_pair().await;
@@ -233,21 +260,22 @@ impl ControllingSelector for AgentInternal {
     }
 
     async fn ping_candidate(
-        &mut self,
+        &self,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
     ) {
         let (msg, result) = {
-            let username = self.remote_ufrag.clone() + ":" + self.local_ufrag.as_str();
+            let ufrag_pwd = self.ufrag_pwd.lock().await;
+            let username = ufrag_pwd.remote_ufrag.clone() + ":" + ufrag_pwd.local_ufrag.as_str();
             let mut msg = Message::new();
             let result = msg.build(&[
                 Box::new(BINDING_REQUEST),
                 Box::new(TransactionId::new()),
                 Box::new(Username::new(ATTR_USERNAME, username)),
-                Box::new(AttrControlling(self.tie_breaker)),
+                Box::new(AttrControlling(self.tie_breaker.load(Ordering::SeqCst))),
                 Box::new(PriorityAttr(local.priority())),
                 Box::new(MessageIntegrity::new_short_term_integrity(
-                    self.remote_pwd.clone(),
+                    ufrag_pwd.remote_pwd.clone(),
                 )),
                 Box::new(FINGERPRINT),
             ]);
@@ -262,13 +290,13 @@ impl ControllingSelector for AgentInternal {
     }
 
     async fn handle_success_response(
-        &mut self,
+        &self,
         m: &Message,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
         remote_addr: SocketAddr,
     ) {
-        if let Some(pending_request) = self.handle_inbound_binding_success(m.transaction_id) {
+        if let Some(pending_request) = self.handle_inbound_binding_success(m.transaction_id).await {
             let transaction_addr = pending_request.destination;
 
             // Assert that NAT is not symmetric
@@ -312,7 +340,7 @@ impl ControllingSelector for AgentInternal {
     }
 
     async fn handle_binding_request(
-        &mut self,
+        &self,
         m: &Message,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
@@ -321,15 +349,20 @@ impl ControllingSelector for AgentInternal {
         log::trace!("controllingSelector: sendBindingSuccess");
 
         if let Some(p) = self.find_pair(local, remote).await {
+            let nominated_pair_is_none = {
+                let nominated_pair = self.nominated_pair.lock().await;
+                nominated_pair.is_none()
+            };
+
             log::trace!(
                 "controllingSelector: after findPair {}, p.state: {}, {}, {}",
                 p,
                 p.state.load(Ordering::SeqCst),
-                self.nominated_pair.is_none(),
+                nominated_pair_is_none,
                 self.agent_conn.get_selected_pair().await.is_none()
             );
             if p.state.load(Ordering::SeqCst) == CandidatePairState::Succeeded as u8
-                && self.nominated_pair.is_none()
+                && nominated_pair_is_none
                 && self.agent_conn.get_selected_pair().await.is_none()
             {
                 if let Some(best_pair) = self.agent_conn.get_best_available_candidate_pair().await {
@@ -343,7 +376,10 @@ impl ControllingSelector for AgentInternal {
                     {
                         log::trace!("The candidate ({}, {}) is the best candidate available, marking it as nominated",
                             p.local, p.remote);
-                        self.nominated_pair = Some(p);
+                        {
+                            let mut nominated_pair = self.nominated_pair.lock().await;
+                            *nominated_pair = Some(p);
+                        }
                         self.nominate_pair().await;
                     }
                 } else {
@@ -359,15 +395,15 @@ impl ControllingSelector for AgentInternal {
 
 #[async_trait]
 impl ControlledSelector for AgentInternal {
-    fn start(&mut self) {}
+    async fn start(&self) {}
 
-    async fn contact_candidates(&mut self) {
+    async fn contact_candidates(&self) {
         // A lite selector should not contact candidates
-        if self.lite {
+        if self.lite.load(Ordering::SeqCst) {
             self.validate_selected_pair().await;
         } else if self.agent_conn.get_selected_pair().await.is_some() {
             if self.validate_selected_pair().await {
-                log::trace!("checking keepalive");
+                log::trace!("[{}]: checking keepalive", self.get_name());
                 self.check_keepalive().await;
             }
         } else {
@@ -376,21 +412,22 @@ impl ControlledSelector for AgentInternal {
     }
 
     async fn ping_candidate(
-        &mut self,
+        &self,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
     ) {
         let (msg, result) = {
-            let username = self.remote_ufrag.clone() + ":" + self.local_ufrag.as_str();
+            let ufrag_pwd = self.ufrag_pwd.lock().await;
+            let username = ufrag_pwd.remote_ufrag.clone() + ":" + ufrag_pwd.local_ufrag.as_str();
             let mut msg = Message::new();
             let result = msg.build(&[
                 Box::new(BINDING_REQUEST),
                 Box::new(TransactionId::new()),
                 Box::new(Username::new(ATTR_USERNAME, username)),
-                Box::new(AttrControlled(self.tie_breaker)),
+                Box::new(AttrControlled(self.tie_breaker.load(Ordering::SeqCst))),
                 Box::new(PriorityAttr(local.priority())),
                 Box::new(MessageIntegrity::new_short_term_integrity(
-                    self.remote_pwd.clone(),
+                    ufrag_pwd.remote_pwd.clone(),
                 )),
                 Box::new(FINGERPRINT),
             ]);
@@ -405,7 +442,7 @@ impl ControlledSelector for AgentInternal {
     }
 
     async fn handle_success_response(
-        &mut self,
+        &self,
         m: &Message,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
@@ -417,7 +454,7 @@ impl ControlledSelector for AgentInternal {
         // request with an appropriate error code response (e.g., 400)
         // [RFC5389].
 
-        if let Some(pending_request) = self.handle_inbound_binding_success(m.transaction_id) {
+        if let Some(pending_request) = self.handle_inbound_binding_success(m.transaction_id).await {
             let transaction_addr = pending_request.destination;
 
             // Assert that NAT is not symmetric
@@ -451,7 +488,7 @@ impl ControlledSelector for AgentInternal {
     }
 
     async fn handle_binding_request(
-        &mut self,
+        &self,
         m: &Message,
         local: &Arc<dyn Candidate + Send + Sync>,
         remote: &Arc<dyn Candidate + Send + Sync>,
