@@ -25,16 +25,19 @@ async fn test_pair_search() -> Result<()> {
     let a = Agent::new(config).await?;
 
     {
-        let ai = a.agent_internal.lock().await;
         {
-            let checklist = ai.agent_conn.checklist.lock().await;
+            let checklist = a.internal.agent_conn.checklist.lock().await;
             assert!(
                 checklist.is_empty(),
                 "TestPairSearch is only a valid test if a.validPairs is empty on construction"
             );
         }
 
-        let cp = ai.agent_conn.get_best_available_candidate_pair().await;
+        let cp = a
+            .internal
+            .agent_conn
+            .get_best_available_candidate_pair()
+            .await;
         assert!(cp.is_none(), "No Candidate pairs should exist");
     }
 
@@ -122,18 +125,24 @@ async fn test_pair_priority() -> Result<()> {
     ];
 
     {
-        let mut ai = a.agent_internal.lock().await;
         for remote in remotes {
-            if ai.find_pair(&host_local, &remote).await.is_none() {
-                ai.add_pair(host_local.clone(), remote.clone()).await;
+            if a.internal.find_pair(&host_local, &remote).await.is_none() {
+                a.internal
+                    .add_pair(host_local.clone(), remote.clone())
+                    .await;
             }
 
-            if let Some(p) = ai.find_pair(&host_local, &remote).await {
+            if let Some(p) = a.internal.find_pair(&host_local, &remote).await {
                 p.state
                     .store(CandidatePairState::Succeeded as u8, Ordering::SeqCst);
             }
 
-            if let Some(best_pair) = ai.agent_conn.get_best_available_candidate_pair().await {
+            if let Some(best_pair) = a
+                .internal
+                .agent_conn
+                .get_best_available_candidate_pair()
+                .await
+            {
                 assert_eq!(
                     best_pair.to_string(),
                     CandidatePair {
@@ -202,10 +211,7 @@ async fn test_on_selected_candidate_pair_change() -> Result<()> {
         Arc::new(relay_remote),
         false,
     ));
-    {
-        let mut ai = a.agent_internal.lock().await;
-        ai.set_selected_pair(Some(p)).await;
-    }
+    a.internal.set_selected_pair(Some(p)).await;
 
     // ensure that the callback fired on setting the pair
     let _ = callback_called_rx.recv().await;
@@ -234,12 +240,11 @@ async fn test_handle_peer_reflexive_udp_pflx_candidate() -> Result<()> {
     let remote = SocketAddr::from_str("172.17.0.3:999")?;
 
     let (username, local_pwd, tie_breaker) = {
-        let ai = a.agent_internal.lock().await;
-
+        let ufrag_pwd = a.internal.ufrag_pwd.lock().await;
         (
-            ai.local_ufrag.to_owned() + ":" + ai.remote_ufrag.as_str(),
-            ai.local_pwd.clone(),
-            ai.tie_breaker,
+            ufrag_pwd.local_ufrag.to_owned() + ":" + ufrag_pwd.remote_ufrag.as_str(),
+            ufrag_pwd.local_pwd.clone(),
+            a.internal.tie_breaker.load(Ordering::SeqCst),
         )
     };
 
@@ -256,18 +261,18 @@ async fn test_handle_peer_reflexive_udp_pflx_candidate() -> Result<()> {
     ])?;
 
     {
-        let mut ai = a.agent_internal.lock().await;
-        ai.handle_inbound(&mut msg, &local, remote).await;
+        a.internal.handle_inbound(&mut msg, &local, remote).await;
 
+        let remote_candidates = a.internal.remote_candidates.lock().await;
         // length of remote candidate list must be one now
         assert_eq!(
-            ai.remote_candidates.len(),
+            remote_candidates.len(),
             1,
             "failed to add a network type to the remote candidate list"
         );
 
         // length of remote candidate list for a network type must be 1
-        if let Some(cands) = ai.remote_candidates.get(&local.network_type()) {
+        if let Some(cands) = remote_candidates.get(&local.network_type()) {
             assert_eq!(
                 cands.len(),
                 1,
@@ -305,14 +310,17 @@ async fn test_handle_peer_reflexive_unknown_remote() -> Result<()> {
     tid.0[..3].copy_from_slice("ABC".as_bytes());
 
     let remote_pwd = {
-        let mut ai = a.agent_internal.lock().await;
-        ai.pending_binding_requests = vec![BindingRequest {
-            timestamp: Instant::now(),
-            transaction_id: tid,
-            destination: SocketAddr::from_str("0.0.0.0:0")?,
-            is_use_candidate: false,
-        }];
-        ai.remote_pwd.clone()
+        {
+            let mut pending_binding_requests = a.internal.pending_binding_requests.lock().await;
+            *pending_binding_requests = vec![BindingRequest {
+                timestamp: Instant::now(),
+                transaction_id: tid,
+                destination: SocketAddr::from_str("0.0.0.0:0")?,
+                is_use_candidate: false,
+            }];
+        }
+        let ufrag_pwd = a.internal.ufrag_pwd.lock().await;
+        ufrag_pwd.remote_pwd.clone()
     };
 
     let host_config = CandidateHostConfig {
@@ -339,11 +347,11 @@ async fn test_handle_peer_reflexive_unknown_remote() -> Result<()> {
     ])?;
 
     {
-        let mut ai = a.agent_internal.lock().await;
-        ai.handle_inbound(&mut msg, &local, remote).await;
+        a.internal.handle_inbound(&mut msg, &local, remote).await;
 
+        let remote_candidates = a.internal.remote_candidates.lock().await;
         assert_eq!(
-            ai.remote_candidates.len(),
+            remote_candidates.len(),
             0,
             "unknown remote was able to create a candidate"
         );
@@ -633,33 +641,45 @@ async fn test_inbound_validity() -> Result<()> {
         let a = Agent::new(AgentConfig::default()).await?;
 
         {
-            let mut ai = a.agent_internal.lock().await;
+            let local_pwd = {
+                let ufrag_pwd = a.internal.ufrag_pwd.lock().await;
+                ufrag_pwd.local_pwd.clone()
+            };
+            a.internal
+                .handle_inbound(
+                    &mut build_msg(CLASS_REQUEST, "invalid".to_owned(), local_pwd)?,
+                    &local,
+                    remote,
+                )
+                .await;
+            {
+                let remote_candidates = a.internal.remote_candidates.lock().await;
+                assert_ne!(
+                    remote_candidates.len(),
+                    1,
+                    "Binding with invalid Username was able to create prflx candidate"
+                );
+            }
 
-            let local_pwd = ai.local_pwd.clone();
-            ai.handle_inbound(
-                &mut build_msg(CLASS_REQUEST, "invalid".to_owned(), local_pwd)?,
-                &local,
-                remote,
-            )
-            .await;
-            assert_ne!(
-                ai.remote_candidates.len(),
-                1,
-                "Binding with invalid Username was able to create prflx candidate"
-            );
-
-            let username = format!("{}:{}", ai.local_ufrag, ai.remote_ufrag);
-            ai.handle_inbound(
-                &mut build_msg(CLASS_REQUEST, username, "Invalid".to_owned())?,
-                &local,
-                remote,
-            )
-            .await;
-            assert_ne!(
-                ai.remote_candidates.len(),
-                1,
-                "Binding with invalid MessageIntegrity was able to create prflx candidate"
-            );
+            let username = {
+                let ufrag_pwd = a.internal.ufrag_pwd.lock().await;
+                format!("{}:{}", ufrag_pwd.local_ufrag, ufrag_pwd.remote_ufrag)
+            };
+            a.internal
+                .handle_inbound(
+                    &mut build_msg(CLASS_REQUEST, username, "Invalid".to_owned())?,
+                    &local,
+                    remote,
+                )
+                .await;
+            {
+                let remote_candidates = a.internal.remote_candidates.lock().await;
+                assert_ne!(
+                    remote_candidates.len(),
+                    1,
+                    "Binding with invalid MessageIntegrity was able to create prflx candidate"
+                );
+            }
         }
 
         a.close().await?;
@@ -670,20 +690,25 @@ async fn test_inbound_validity() -> Result<()> {
         let a = Agent::new(AgentConfig::default()).await?;
 
         {
-            let mut ai = a.agent_internal.lock().await;
-
-            let username = format!("{}:{}", ai.local_ufrag, ai.remote_ufrag);
-            ai.handle_inbound(
-                &mut build_msg(CLASS_SUCCESS_RESPONSE, username, "Invalid".to_owned())?,
-                &local,
-                remote,
-            )
-            .await;
-            assert_ne!(
-                ai.remote_candidates.len(),
-                1,
-                "Binding with invalid Username was able to create prflx candidate"
-            );
+            let username = {
+                let ufrag_pwd = a.internal.ufrag_pwd.lock().await;
+                format!("{}:{}", ufrag_pwd.local_ufrag, ufrag_pwd.remote_ufrag)
+            };
+            a.internal
+                .handle_inbound(
+                    &mut build_msg(CLASS_SUCCESS_RESPONSE, username, "Invalid".to_owned())?,
+                    &local,
+                    remote,
+                )
+                .await;
+            {
+                let remote_candidates = a.internal.remote_candidates.lock().await;
+                assert_ne!(
+                    remote_candidates.len(),
+                    1,
+                    "Binding with invalid Username was able to create prflx candidate"
+                );
+            }
         }
 
         a.close().await?;
@@ -694,17 +719,20 @@ async fn test_inbound_validity() -> Result<()> {
         let a = Agent::new(AgentConfig::default()).await?;
 
         {
-            let mut ai = a.agent_internal.lock().await;
-
-            let username = format!("{}:{}", ai.local_ufrag, ai.remote_ufrag);
-            ai.handle_inbound(
-                &mut build_msg(CLASS_ERROR_RESPONSE, username, "Invalid".to_owned())?,
-                &local,
-                remote,
-            )
-            .await;
+            let username = {
+                let ufrag_pwd = a.internal.ufrag_pwd.lock().await;
+                format!("{}:{}", ufrag_pwd.local_ufrag, ufrag_pwd.remote_ufrag)
+            };
+            a.internal
+                .handle_inbound(
+                    &mut build_msg(CLASS_ERROR_RESPONSE, username, "Invalid".to_owned())?,
+                    &local,
+                    remote,
+                )
+                .await;
+            let remote_candidates = a.internal.remote_candidates.lock().await;
             assert_ne!(
-                ai.remote_candidates.len(),
+                remote_candidates.len(),
                 1,
                 "non-binding message was able to create prflxRemote"
             );
@@ -718,18 +746,23 @@ async fn test_inbound_validity() -> Result<()> {
         let a = Agent::new(AgentConfig::default()).await?;
 
         {
-            let mut ai = a.agent_internal.lock().await;
-
-            let username = format!("{}:{}", ai.local_ufrag, ai.remote_ufrag);
-            let local_pwd = ai.local_pwd.clone();
-            ai.handle_inbound(
-                &mut build_msg(CLASS_REQUEST, username, local_pwd)?,
-                &local,
-                remote,
-            )
-            .await;
+            let (username, local_pwd) = {
+                let ufrag_pwd = a.internal.ufrag_pwd.lock().await;
+                (
+                    format!("{}:{}", ufrag_pwd.local_ufrag, ufrag_pwd.remote_ufrag),
+                    ufrag_pwd.local_pwd.clone(),
+                )
+            };
+            a.internal
+                .handle_inbound(
+                    &mut build_msg(CLASS_REQUEST, username, local_pwd)?,
+                    &local,
+                    remote,
+                )
+                .await;
+            let remote_candidates = a.internal.remote_candidates.lock().await;
             assert_eq!(
-                ai.remote_candidates.len(),
+                remote_candidates.len(),
                 1,
                 "Binding with valid values was unable to create prflx candidate"
             );
@@ -743,10 +776,13 @@ async fn test_inbound_validity() -> Result<()> {
         let a = Agent::new(AgentConfig::default()).await?;
 
         {
-            let mut ai = a.agent_internal.lock().await;
-
-            let username = format!("{}:{}", ai.local_ufrag, ai.remote_ufrag);
-            let local_pwd = ai.local_pwd.clone();
+            let (username, local_pwd) = {
+                let ufrag_pwd = a.internal.ufrag_pwd.lock().await;
+                (
+                    format!("{}:{}", ufrag_pwd.local_ufrag, ufrag_pwd.remote_ufrag),
+                    ufrag_pwd.local_pwd.clone(),
+                )
+            };
 
             let mut msg = Message::new();
             msg.build(&[
@@ -756,9 +792,10 @@ async fn test_inbound_validity() -> Result<()> {
                 Box::new(MessageIntegrity::new_short_term_integrity(local_pwd)),
             ])?;
 
-            ai.handle_inbound(&mut msg, &local, remote).await;
+            a.internal.handle_inbound(&mut msg, &local, remote).await;
+            let remote_candidates = a.internal.remote_candidates.lock().await;
             assert_eq!(
-                ai.remote_candidates.len(),
+                remote_candidates.len(),
                 1,
                 "Binding with valid values (but no fingerprint) was unable to create prflx candidate"
             );
@@ -772,13 +809,15 @@ async fn test_inbound_validity() -> Result<()> {
         let a = Agent::new(AgentConfig::default()).await?;
 
         {
-            let mut ai = a.agent_internal.lock().await;
             let remote = SocketAddr::from_str("172.17.0.3:999")?;
 
             let mut t_id = TransactionId::default();
             t_id.0[..3].copy_from_slice(b"ABC");
 
-            let remote_pwd = ai.remote_pwd.clone();
+            let remote_pwd = {
+                let ufrag_pwd = a.internal.ufrag_pwd.lock().await;
+                ufrag_pwd.remote_pwd.clone()
+            };
 
             let mut msg = Message::new();
             msg.build(&[
@@ -788,12 +827,16 @@ async fn test_inbound_validity() -> Result<()> {
                 Box::new(FINGERPRINT),
             ])?;
 
-            ai.handle_inbound(&mut msg, &local, remote).await;
-            assert_eq!(
-                ai.remote_candidates.len(),
-                0,
-                "unknown remote was able to create a candidate"
-            );
+            a.internal.handle_inbound(&mut msg, &local, remote).await;
+
+            {
+                let remote_candidates = a.internal.remote_candidates.lock().await;
+                assert_eq!(
+                    remote_candidates.len(),
+                    0,
+                    "unknown remote was able to create a candidate"
+                );
+            }
         }
 
         a.close().await?;
@@ -1067,18 +1110,17 @@ async fn test_candidate_pair_stats() -> Result<()> {
         Arc::clone(&prflx_remote),
         Arc::clone(&host_remote),
     ] {
-        let mut ai = a.agent_internal.lock().await;
-        let p = ai.find_pair(&host_local, remote).await;
+        let p = a.internal.find_pair(&host_local, remote).await;
 
         if p.is_none() {
-            ai.add_pair(Arc::clone(&host_local), Arc::clone(remote))
+            a.internal
+                .add_pair(Arc::clone(&host_local), Arc::clone(remote))
                 .await;
         }
     }
 
     {
-        let ai = a.agent_internal.lock().await;
-        if let Some(p) = ai.find_pair(&host_local, &prflx_remote).await {
+        if let Some(p) = a.internal.find_pair(&host_local, &prflx_remote).await {
             p.state
                 .store(CandidatePairState::Failed as u8, Ordering::SeqCst);
         }
@@ -1182,8 +1224,8 @@ async fn test_local_candidate_stats() -> Result<()> {
     );
 
     {
-        let mut ai = a.agent_internal.lock().await;
-        ai.local_candidates.insert(
+        let mut local_candidates = a.internal.local_candidates.lock().await;
+        local_candidates.insert(
             NetworkType::Udp4,
             vec![Arc::clone(&host_local), Arc::clone(&srflx_local)],
         );
@@ -1308,8 +1350,8 @@ async fn test_remote_candidate_stats() -> Result<()> {
     );
 
     {
-        let mut ai = a.agent_internal.lock().await;
-        ai.remote_candidates.insert(
+        let mut remote_candidates = a.internal.remote_candidates.lock().await;
+        remote_candidates.insert(
             NetworkType::Udp4,
             vec![
                 Arc::clone(&relay_remote),
@@ -1498,26 +1540,31 @@ async fn test_binding_request_timeout() -> Result<()> {
 
     let now = Instant::now();
     {
-        let mut ai = a.agent_internal.lock().await;
-        ai.pending_binding_requests.push(BindingRequest {
-            timestamp: now, // valid
-            ..Default::default()
-        });
-        ai.pending_binding_requests.push(BindingRequest {
-            timestamp: now.sub(Duration::from_millis(3900)), // valid
-            ..Default::default()
-        });
-        ai.pending_binding_requests.push(BindingRequest {
-            timestamp: now.sub(Duration::from_millis(4100)), // invalid
-            ..Default::default()
-        });
-        ai.pending_binding_requests.push(BindingRequest {
-            timestamp: now.sub(Duration::from_secs(75)), // invalid
-            ..Default::default()
-        });
+        {
+            let mut pending_binding_requests = a.internal.pending_binding_requests.lock().await;
+            pending_binding_requests.push(BindingRequest {
+                timestamp: now, // valid
+                ..Default::default()
+            });
+            pending_binding_requests.push(BindingRequest {
+                timestamp: now.sub(Duration::from_millis(3900)), // valid
+                ..Default::default()
+            });
+            pending_binding_requests.push(BindingRequest {
+                timestamp: now.sub(Duration::from_millis(4100)), // invalid
+                ..Default::default()
+            });
+            pending_binding_requests.push(BindingRequest {
+                timestamp: now.sub(Duration::from_secs(75)), // invalid
+                ..Default::default()
+            });
+        }
 
-        ai.invalidate_pending_binding_requests(now);
-        assert_eq!(EXPECTED_REMOVAL_COUNT, ai.pending_binding_requests.len(), "Binding invalidation due to timeout did not remove the correct number of binding requests")
+        a.internal.invalidate_pending_binding_requests(now).await;
+        {
+            let pending_binding_requests = a.internal.pending_binding_requests.lock().await;
+            assert_eq!(EXPECTED_REMOVAL_COUNT, pending_binding_requests.len(), "Binding invalidation due to timeout did not remove the correct number of binding requests")
+        }
     }
 
     a.close().await?;
@@ -1534,9 +1581,9 @@ async fn test_agent_credentials() -> Result<()> {
 
     let a = Agent::new(AgentConfig::default()).await?;
     {
-        let ai = a.agent_internal.lock().await;
-        assert!(ai.local_ufrag.as_bytes().len() * 8 >= 24);
-        assert!(ai.local_pwd.as_bytes().len() * 8 >= 128);
+        let ufrag_pwd = a.internal.ufrag_pwd.lock().await;
+        assert!(ufrag_pwd.local_ufrag.as_bytes().len() * 8 >= 24);
+        assert!(ufrag_pwd.local_pwd.as_bytes().len() * 8 >= 128);
     }
     a.close().await?;
 
@@ -1613,9 +1660,14 @@ async fn test_connection_state_failed_delete_all_candidates() -> Result<()> {
     let _ = is_failed_rx.recv().await;
 
     {
-        let ai = a_agent.agent_internal.lock().await;
-        assert_eq!(ai.remote_candidates.len(), 0);
-        assert_eq!(ai.local_candidates.len(), 0);
+        {
+            let remote_candidates = a_agent.internal.remote_candidates.lock().await;
+            assert_eq!(remote_candidates.len(), 0);
+        }
+        {
+            let local_candidates = a_agent.internal.local_candidates.lock().await;
+            assert_eq!(local_candidates.len(), 0);
+        }
     }
 
     a_agent.close().await?;
@@ -1872,10 +1924,13 @@ async fn test_get_remote_credentials() -> Result<()> {
     let a = Agent::new(AgentConfig::default()).await?;
 
     let (remote_ufrag, remote_pwd) = {
-        let mut ai = a.agent_internal.lock().await;
-        ai.remote_ufrag = "remoteUfrag".to_owned();
-        ai.remote_pwd = "remotePwd".to_owned();
-        (ai.remote_ufrag.to_owned(), ai.remote_pwd.to_owned())
+        let mut ufrag_pwd = a.internal.ufrag_pwd.lock().await;
+        ufrag_pwd.remote_ufrag = "remoteUfrag".to_owned();
+        ufrag_pwd.remote_pwd = "remotePwd".to_owned();
+        (
+            ufrag_pwd.remote_ufrag.to_owned(),
+            ufrag_pwd.remote_pwd.to_owned(),
+        )
     };
 
     let (actual_ufrag, actual_pwd) = a.get_remote_user_credentials().await;
